import math;
from .geoOperation import getGeoInfo;
from shapely.geometry import LineString;
from shapely.geometry import Point;
from shapely.geometry import Polygon;

class DistanceOperation():
    def getDifClassDotDistance(self, dot1, dot2):
        return math.sqrt((dot1[0] - dot2[0])*(dot1[0] - dot2[0]) + (dot1[1] - dot2[1])*(dot1[1] - dot2[1]))

class WingletsStepHandler():
    m_mapDisMatrix = {}
    m_mapClassIdDotIndexSihouttle = {}
    m_mapClassIdMaxMinSI = {}
    m_mapClassIdDotIndexSensitivity = {}
    m_drawInterContour = {}
    m_mapClassIdMaxMinLength = {}
    m_mapClassIdDotIndexStroke = {}
    m_mapClassIdDotIndexStrokeBag = {}
    m_WC_ClassIDPointDis_minmax = {}
    m_test_mainContour = {}

    m_xMin = 0
    m_xMax = 0
    m_yMin = 0
    m_yMax = 0
    m_OC_PointDisToCentroid_min = 1e6
    m_OC_PointDisToCentroid_max = -1e6
    m_diff = True

    geoInstance = getGeoInfo()

    def dot2Canvas(self, dot, scale=8):
        
        return [scale * dot[0], scale * dot]
    
    def convertContourToCanvas(self, mapIsovalueContours):
        mapIsovalueCanvasContours = {}
        for Isovalue in mapIsovalueContours:
            contours = mapIsovalueContours[Isovalue]
            for i in range(len(contours)):
                contour = contours[i]
                canvasContour = []
                for p in range(len(contour)):
                    pos = contour[p]
                    canvasContour.append(self.dot2Canvas([pos[1], 100 - pos[0]]))
                    if Isovalue not in mapIsovalueCanvasContours.keys():
                        mapIsovalueCanvasContours[Isovalue] = [canvasContour]
                    else:
                        mapIsovalueCanvasContours[Isovalue].append(canvasContour)
        return mapIsovalueCanvasContours

    # 不知道有啥用
    def sortIsovalue(self, mapIsovalueContour):
        liSortedIsovalueStr = []
        liIsovalueStr = list(mapIsovalueContour.keys())
        liNewIsovalue = []

        for i in range(len(liIsovalueStr)):
            # liNewIsovalue.append(int(liIsovalueStr[i]))
            liNewIsovalue.append(float(liIsovalueStr[i]))
        liNewIsovalue.sort()

        for i in range(len(liNewIsovalue)):
            curIsovalueNum = liNewIsovalue[i]
            for j in range(len(liIsovalueStr)):
                # IsovalueTmp = int(liIsovalueStr[j])
                IsovalueTmp = float(liIsovalueStr[j])
                if(abs(curIsovalueNum - IsovalueTmp) < 1e-6):
                    liSortedIsovalueStr.append(liIsovalueStr[j])
        return liSortedIsovalueStr

    def getRadialIntersection(self, pos1, pos2, contour):
        centroidPos = Point(pos1[0], pos1[1])
        # centroidPos = Point(1500 * pos1[0] - 1500 * pos2[0], 1500 * pos1[1] - 1500 * pos2[1])
        dataPos = Point(3000 * pos2[0] - 3000 * pos1[0] + pos1[0], 3000 * pos2[1] - 3000 * pos1[1] + pos1[1])
        # dataPos = Point(1500 * pos2[0], 1500 * pos2[1])
        # dataPos = Point(pos2[0] - pos1[0], pos2[1] - pos1[1])
        centroidAlongLinePath = LineString([centroidPos, dataPos])

        contourPathPoints = []
        for i in range(len(contour)):
            contourPathPoints.append(Point(contour[i][0], contour[i][1]))
        # 因为shapely没有添加的api，所以将点放在数组里使用Polygon一次性生成polygon
        contourPath = Polygon(contourPathPoints)

        # 通过shapely的intersection获取到LineString和Polygon的交集，应该是一条lineString，可以用coords来获取交集的点
        # 如果交集为一个线段，一个点为交点，一个点为线段中的一个点，一般是第一个元素
        # liIntersectionsCoords = list(contourPath.intersection(centroidAlongLinePath).coords)
        # print('test', centroidAlongLinePath.intersection(contourPath))
        liIntersections = centroidAlongLinePath.intersection(contourPath)
        liIntersectionsCoords = []
        if (liIntersections.type == 'MultiLineString'):
            print('multiLineString')
            listLiIntersections = list(liIntersections)
            for i in range(len(listLiIntersections)):
                if i == 0:
                    curIntersectionLineStringCoords = list(listLiIntersections[i].coords)
                    print('curDot1', centroidPos)
                    print('pos2', pos2)
                    print('curDot2', dataPos)
                    print('curIntersectionLineStringCoords', curIntersectionLineStringCoords)
                    # liIntersectionsCoords.append(curIntersectionLineStringCoords[0])
                    liIntersectionsCoords.append(curIntersectionLineStringCoords[1])
                # liIntersectionsCoords.append(list(listLiIntersections[i].coords))
            # liIntersectionsCoords= list(listLiIntersections[0].coords)
        else:
            # liIntersectionsCoords = list(centroidAlongLinePath.intersection(contourPath).coords)
            liIntersectionsCoords = list(centroidAlongLinePath.intersection(contourPath).coords)
            liIntersectionsCoords = [liIntersectionsCoords[1]]
            # print('liIntersectionsCoords', liIntersectionsCoords)
            # print('liIntersectionsCoords[1]', liIntersectionsCoords[1])

        #chose the closest one
        minDistance = 1e6
        minIndex = -1
        for i in range(len(liIntersectionsCoords)):
            dis_temp = math.sqrt(math.pow(liIntersectionsCoords[i][0] - pos2[0], 2) + math.pow(liIntersectionsCoords[i][1] - pos2[1], 2))
            if (dis_temp < minDistance):
                minDistance = dis_temp
                minIndex = i
        if (minIndex == -1):
            return {'pos': [], 'index': -1}    
    	
        minDistanceOnContour = 1e6
        minIndexOnContour = -1
        curIntersectionPos = [liIntersectionsCoords[minIndex][0], liIntersectionsCoords[minIndex][1]]
        for i in range(len(contour)):
            dis_temp_onContour = math.sqrt(math.pow(curIntersectionPos[0] - contour[i][0], 2) + math.pow(curIntersectionPos[1] - contour[i][1], 2))
            if dis_temp_onContour < minDistanceOnContour:
                minIndexOnContour = i
                minDistanceOnContour = dis_temp_onContour

        # if minDistanceOnContour < 5:
        #     return {
        #         # 'pos': [liIntersectionsCoords[minIndex][0], liIntersectionsCoords[minIndex][1]],
        #         'pos': curIntersectionPos,
        #         #? intersections[minIndex].index
        #         'index': minIndexOnContour
        #     }
        # else:
        #     print('curDistance > 1', minDistanceOnContour)
        # if minDistanceOnContour > 3:
        #     print('curDistance > 1', minDistanceOnContour)

        # contour.insert(minIndexOnContour, curIntersectionPos)

        return {
            # 'pos': [liIntersectionsCoords[minIndex][0], liIntersectionsCoords[minIndex][1]],
            'pos': curIntersectionPos,
            #? intersections[minIndex].index
            'index': minIndexOnContour
            }
        



    def interpolateContourSelf(self, contour, contourNum):
        mapIndexContour = {}
        liDeformVector = []
        sampleNum = 120
        # print('contour', contour)
        # 算出当前contour的centroid，此时的contour为前几步找到的当前class中的应该保留的外层公共contour
        centroidPos = self.geoInstance.getCentroid(contour)
        # 希望的是将当前公共contour分成120份，对每一份求交点来对每一份进行插值 ？分成120份的意义
        for i in range(sampleNum):
            # 这个arc就可以在下面帮助生成中心点在当前i份中(120份中的第i份)发出的射线
            arc = (i * 2 * math.pi) / sampleNum
            intersectResult_outer = self.getRadialIntersection(centroidPos, [centroidPos[0] + 100*math.cos(arc), centroidPos[1] + 100*math.sin(arc)], contour)
            deformVector = [centroidPos, intersectResult_outer['pos']]
            liDeformVector.append(deformVector)
            #interpolate
            #想要interpolate出contourNum个contour
            for p in range(contourNum):
                # percentage为当前的percentage，contourPos为根据当前的percentage与当前弧度生成的deformVector代表的向量所插成的点
                perCentage = (p+1) / (contourNum + 1)
                contourPos = self.geoInstance.getLineXYatPercent({'x': deformVector[0][0], 'y': deformVector[0][1]},
                                                               {'x': deformVector[1][0], 'y': deformVector[1][1]},
                                                               perCentage)

                # mapIndexContour为 {}，以p也就是contourNum遍历的index作为key，也就是第几层contour，
                # 而在不同的p层contour中append的为遍历了sampleNum次生成的不同弧度对应的contourPos。

                # 即每一个对象的key对应了该contour下sampleNum个数组，
                # deformVector：centroids与该点生成的向量1与最近的contour生成的交集，然后中心点和该交集共同作为deformVector
                # 通过循环了contourNum次，找到每个contourNum下的deformVector的点，并将这些点放置在对应的key为p的数组里面，
                # 每个p对应的数组里存放了 sampleNum个 交点
                if(p not in mapIndexContour.keys()):
                    mapIndexContour[p] = [[contourPos['x'], contourPos['y']]]
                else:
                    mapIndexContour[p].append([contourPos['x'], contourPos['y']])
        return mapIndexContour

    def getLengthBySI(self, curSI):
        return 9 + curSI * 18

    # ? spcanvas.js第3211行，有很多判断，只选取了 gestalt的
    def getLengthSpecialCase(self, curCurveLength, curSI):
        s = 8
        # ? 本来是15, 但是太短了
        curveLength = s + curSI * 18
        if curveLength < s:
            curveLength = s
        return curveLength
    
    def adjustStrokesofClass(self, mapIndexStrokeBag):
        mapIndexCurve = {}
        # finer the curve and adapt to contourIndex
        liDotIndex = list(mapIndexStrokeBag.keys())
        m_liVisibleDotIndex = []

        for i in range(len(liDotIndex)):
            curDotIndex = liDotIndex[i]
            curveBag = mapIndexStrokeBag[curDotIndex]
            dotType = curveBag['dotType']
            sortIsovalueIndex = curveBag['sortIsovalueIndex']
            centroid = curveBag['centroid']
            liIntersectPos = curveBag['intersection']
            baseDot = curveBag['baseDot']
            curveLength = self.geoInstance.getLengthbyContourIndex(sortIsovalueIndex)
            newCurve = curveBag['dots']

            mapIndexCurve[curDotIndex] = {
                'originDot': curveBag['originDot'],
                'dotType': dotType,
                'curve': newCurve,
                'level': sortIsovalueIndex,
                'centroid': centroid,
                'intersection': liIntersectPos,
                'baseDot': baseDot
            }
        return mapIndexCurve

    def preLocate(self, dots, liSihoutteIndex, liSensitivity, liKDE, 
        mapIsovalueCanvasContour, mapIsovalueInterContours, liSortedIsovalueStr):
        pointToCentroid_min = 1e6
        pointToCentroid_max = -1e6
        liIsovalue = list(mapIsovalueCanvasContour.keys())  #?获得的应该是最外层contour的Isovalue，因为mapIsovalueCanvasContour只保留了outContour

        mapIsovalueIndexContour = {}
        mapIsovalueStrCentroids = {}

        # compute the centroid of contours，计算的是 每个Isovalue下每个contour的centroid ？但是mapIsovalueCanvasContour里面放的是外层的公共contour
        for i in range(len(liIsovalue)):
            liContour_temp = mapIsovalueCanvasContour[liIsovalue[i]]
            for j in range(len(liContour_temp)):
                contour = liContour_temp[j]
                centroid = self.geoInstance.getCentroid(contour)
                if(liIsovalue[i] not in mapIsovalueStrCentroids.keys()):
                    mapIsovalueStrCentroids[liIsovalue[i]] = [centroid]
                else:
                    mapIsovalueStrCentroids[liIsovalue[i]].append(centroid)
        
        mapIndexCurve = {}
        mapIndexStrokeBag = {}

        step = (self.m_xMax - self.m_xMin) / 100    # step = 8
        mapDotIndexIsoBag = {}
        minLength = 1e6
        maxLength = -1e6

        # get the closest minIsovalue_str and minContourIndex
        minLowIsoIndex = 1e6
        for i in range(len(dots)):
            curDot = dots[i]
            # 确定当前点应该在800x800的哪个格子里（以8为单位）
            cellColumnIndex = math.floor((curDot[0] - self.m_xMin) / step)
            cellRowIndex = math.floor((self.m_yMax - curDot[1]) / step)
            density = liKDE[cellRowIndex * 100 + cellColumnIndex]

            # get the two boundary
            lowIsovalueStr = upperIsovalueStr = None
            lowIsoIndex = upperIsoIndex = -1
            #？liSortedIsovalueStr里只有[outIsovalue]，这个循环会跳过
            for j in range(len(liSortedIsovalueStr) - 1):
                Isovalue1 = float(liSortedIsovalueStr[j])
                Isovalue2 = float(liSortedIsovalueStr[j + 1])
                if(Isovalue1 <= density and Isovalue2 >= density):
                    lowIsovalueStr = liSortedIsovalueStr[j]
                    upperIsovalueStr = liSortedIsovalueStr[j+1]
                    lowIsoIndex = j
                    upperIsoIndex = j+1
                    if(lowIsoIndex < minLowIsoIndex):
                        minLowIsoIndex = lowIsoIndex

            #? 得清楚这几个是干嘛的, 是为了记录离点最近的isovalue和isovalue对应的contour的index嘛 因为上面的循环不能运行，lowIsoIndex = 0，lowIsovalueStr为最外层contour的Isovalue，upperIsoIndex=1，minLowIsoIndex = 0
            if lowIsoIndex == -1:
                lowIsoIndex = len(liSortedIsovalueStr) - 1
                lowIsovalueStr = liSortedIsovalueStr[lowIsoIndex]
                upperIsovalueStr = None
                upperIsoIndex = lowIsoIndex + 1
                if lowIsoIndex < minLowIsoIndex:
                    minLowIsoIndex = lowIsoIndex
            mapDotIndexIsoBag[i] = {
                'lowIsoIndex': lowIsoIndex,
                'lowIsovalueStr': lowIsovalueStr,
                'upperIsoIndex': upperIsoIndex,
                'upperIsovalueStr': upperIsovalueStr
            }

        # 为每个点添加winglets
        # 不知道上面的循环时干嘛的, 都是对 dots遍历, 上面几乎没做什么, 只是将lowIsoIndex等值存入, 且都是初始化时的值
        for i in range(len(dots)):
            curDot = dots[i]
            cellColumnIndex = math.floor((curDot[0] - self.m_xMin) / step)
            cellRowIndex = math.floor((self.m_yMax - curDot[1]) / step)
            # ? 不知道用处
            # overlap = self.m_OC_liOverlap[cellRowIndex * 100 + cellRowIndex]
            sihoutte = None
            closePos = []
            if(len(liSihoutteIndex) > i):
                sihoutte = liSihoutteIndex[i]
            else:
                sihoutte = 1
            
            sensitivity = None
            if(len(liSensitivity) > i):
                sensitivity = liSensitivity[i]
            else:
                sensitivity = 0
            isoBag = mapDotIndexIsoBag[i]
            lowIsoIndex = isoBag['lowIsoIndex']
            lowIsovalueStr = isoBag['lowIsovalueStr']
            upperIsovalueStr = isoBag['upperIsoIndex']
            upperIsoIndex = isoBag['upperIsovalueStr']

            dotType = 'none'
            if lowIsoIndex == minLowIsoIndex:
                dotType = 'boundary'
            else:
                dotType = i


            # get the interpolate isocontour
            # get the closest isocontour
            closeContour = None
            closePointIndex = -1
            liCandiateContour = []
            for p in range(len(mapIsovalueCanvasContour[lowIsovalueStr])):
                liCandiateContour.append(mapIsovalueCanvasContour[lowIsovalueStr][p])
            #？不懂这个地方的用处，
            if(upperIsovalueStr in mapIsovalueCanvasContour.keys()):
                for p in range(len(mapIsovalueCanvasContour[upperIsovalueStr])):
                    liCandiateContour.append(mapIsovalueCanvasContour[upperIsovalueStr][p])
            # 将插值得到的contour中最外层Isovalue值得contour赋给了mapInterContour，并把个Isovalue下得contour添加在了liCandiateContour，？感觉不对
            mapInterContours = mapIsovalueInterContours[lowIsovalueStr]
            for keys_temp in mapInterContours:
                liCandiateContour.append(mapInterContours[keys_temp])
            
            # print('liCandiateContour len', len(liCandiateContour))
            # if len(liCandiateContour) != 1:
            #     print('liCandiateContour', liCandiateContour)
            
            centroidPos = mapIsovalueStrCentroids[lowIsovalueStr][0]
            minDistance = 1e6

            if lowIsoIndex < 3:
                closeContourIndex = -1
                for j in range(len(liCandiateContour)):
                    curIntersection = self.getRadialIntersection(centroidPos, curDot, liCandiateContour[j])
                    curIntersectPos = curIntersection['pos']
                    distance_temp = (curIntersectPos[0] - curDot[0]) * (curIntersectPos[0] - curDot[0]) + (curIntersectPos[1] - curDot[1]) * (curIntersectPos[1] - curDot[1])
                    if(minDistance > distance_temp):
                        # 只是将contour中值赋值给了closeContour
                        closeContour = liCandiateContour[j][:]
                        closePointIndex = curIntersection['index']
                        closePos = curIntersectPos
                        closeContourIndex = j
                        minDistance = distance_temp
            else:
                for j in range(len(liCandiateContour)):
                    contour_temp = liCandiateContour[j]
                    for p in range(len(contour_temp)):
                        dot_temp = contour_temp[p]
                        distance_temp = self.geoInstance.getDis(dot_temp, curDot)
                        if minDistance > distance_temp:
                            closeContour = contour_temp
                            closePointIndex = p
                            closePos = dot_temp
                            closeContourIndex = j
                            minDistance = distance_temp
            
            # 将 closePos与就是交点插入closeContour里, 改变closePointIndex, 并将随后的index整体加一
            tempLeftIndex = closePointIndex - 1 
            tempRightIndex = closePointIndex + 1 
            if closePointIndex == len(closeContour)-1:
                tempRightIndex = 0
            if closePointIndex == 0:
                tempLeftIndex = len(closeContour)-1
            leftPosToClosePosDis = self.geoInstance.getDis(closeContour[tempLeftIndex], closePos)
            rightPosToClosePosDis = self.geoInstance.getDis(closeContour[tempRightIndex], closePos)
            if leftPosToClosePosDis < rightPosToClosePosDis:
                closeContour.insert(closePointIndex, closePos)
            else:
                closeContour.insert(tempRightIndex, closePos)
                closePointIndex = tempRightIndex

            # ? 这个应该时临时存储每个点 最近的contour
            tempContourKey = str(lowIsoIndex) + '_' + str(closeContourIndex)
            if tempContourKey not in mapIsovalueIndexContour.keys():
                mapIsovalueIndexContour[tempContourKey] = closeContour
            
            # if(dotType == 'boundary'){
            #     this.m_drawInterContour['dots'] = [centroidPos];
            # }
        
            curve = []
            # ？centroid的定义在哪
            disToCentroid = self.geoInstance.getDis(curDot, centroid)

            if(self.m_OC_PointDisToCentroid_min > disToCentroid):		
                self.m_OC_PointDisToCentroid_min = disToCentroid
            if(self.m_OC_PointDisToCentroid_max < disToCentroid):
                self.m_OC_PointDisToCentroid_max = disToCentroid
            
            if(pointToCentroid_min > disToCentroid):		
                pointToCentroid_min = disToCentroid
            if(pointToCentroid_max < disToCentroid):
                pointToCentroid_max = disToCentroid;

            # 这个是循环每个点都会定义的curveBag，为该点接下来画curve做准备，最重要的是后三个属性
            # distanceToCentroid存放当前点到？上面的有问题 311行
            # intersection和intersectionIndex存放离当前点最近的位于contour上的点坐标和closePointIndex = curIntersection['index']/closePointIndex = p
            curveBag = {
                'dotType': dotType,
                'dots': [],
                'longdots': [],
                'originDot': curDot,
                'sortIsovalueIndex': lowIsoIndex,
                'centroid': centroid,
                'sihoutte': sihoutte,
                'sensitivity': sensitivity,
                # 上面注释了所以这儿也注释了
                #? 'overlap': overlap,
                'distanceToCentroid': disToCentroid,
                'intersection': closePos,
                'intersectionIndex': closePointIndex,
            }

            curveLength = None

            # if g_sensityvityRender == False:
            curveLength = self.getLengthBySI(sihoutte)
            # else:
                # curveLength = self.getLengthBySI(sensitivity)
            curveLength = self.getLengthSpecialCase(curveLength, sihoutte)
            # 
            # curveLength = 20

            if(curveLength > maxLength):
                maxLength = curveLength
            if(curveLength < minLength):
                minLength = curveLength
            
            curveBag['strokeLength'] = curveLength
            leftLength = curveLength/2
            rightLength = curveLength/2;

            diff = self.m_diff
            if closeContour is not None:
                if curveBag['dotType'] is not None:
                    # 将最近的点放在当前点的curveBag的baseDot里
                    curveBag['baseDot'] = closePos
                    curBaseDot = curveBag['baseDot']
                    diffVector = [0, 0]
                    if diff:
                        diffVector = [curDot[0] - curBaseDot[0], curDot[1] - curBaseDot[1]]
                    
                    # ? 
                    self.m_drawInterContour['dots1'] = []
                    self.m_drawInterContour['dots2'] = []
                    liLeftDot = []
                    curveBag['dots'] = []

                    if leftLength != 0:
                        # ? 
                        self.m_drawInterContour['reddots'] = [closePos]
                        self.m_drawInterContour['blackdots'] = [curDot]

                        # left 左半部分的winglets, tobeAddLength初始应该是0, 所以有个while判断, 将当前的contour上的点压入drawInterContour的dots1里面,也压入liLeftDot里
                        # 当这个length大于leftLength也就是满足了我们想要的winglets左半部分的长度后就结束循环
                        leftPos = closePos
                        tobeAddPointIndex = closePointIndex
                        tobeAddPos = closeContour[tobeAddPointIndex]
                        tobeAddLength = self.geoInstance.getDis(leftPos, tobeAddPos)
                        while tobeAddLength <= leftLength:
                            self.m_drawInterContour['dots1'].append(tobeAddPos)
                            liLeftDot.append(tobeAddPos)
                            tobeAddPointIndex -= 1
                            if tobeAddPointIndex < 0:
                                # ? name = 4的时候 tobeAddPointIndex = len(closeContour) - 1 和 tobeAddPointIndex = 0时数据点是一样的
                                # tobeAddPointIndex = len(closeContour) - 1
                                print('left: pointIndex < 0')
                                tobeAddPointIndex = len(closeContour) - 1
                                # print('tobeAddPointIndex < 0',tobeAddPointIndex)
                                # print('tobeAddPointIndex[0]',closeContour[0])
                                # print('tobeAddPointIndex[len(closeContour) - 1]',closeContour[len(closeContour) - 1])
                            leftLength -= tobeAddLength
                            leftPos = tobeAddPos
                            tobeAddPos = closeContour[tobeAddPointIndex]
                            tobeAddLength = self.geoInstance.getDis(leftPos, tobeAddPos)

                        if leftLength < tobeAddLength:
                            # print('leftPos', leftPos)
                            # print('tobeAddPointIndex',tobeAddPointIndex)
                            interPos = self.geoInstance.getPosinLine(leftPos, tobeAddPos, leftLength)
                            self.m_drawInterContour['dots1'].append(interPos)
                            liLeftDot.append(interPos)
                        for temp in range(len(liLeftDot)-1, -1, -1):
                            curveBag['dots'].append([liLeftDot[temp][0] + diffVector[0], liLeftDot[temp][1] + diffVector[1]])

                    if diff:
                        curveBag['dots'].append(curDot)
                    else:
                        curveBag['dots'].append(closePos)

                    # right
                    # if rightLength != 0:
                    #     rightPos = closePos
                    #     tobeAddPointIndex = closePointIndex + 1
                    #     if tobeAddPointIndex >= len(closeContour):
                    #         tobeAddPointIndex = 0
                    #     tobeAddPos = closeContour[tobeAddPointIndex]
                    #     # if tobeAddPos is None:
                    #     tobeAddLength = self.geoInstance.getDis(rightPos, tobeAddPos)
                    #     while tobeAddLength <= rightLength:
                    #         self.m_drawInterContour['dots2'].append(tobeAddPos)
                    #         curveBag['dots'].append([tobeAddPos[0] + diffVector[0], tobeAddPos[1] + diffVector[1]])
                    #         tobeAddPointIndex += 1
                    #         if tobeAddPointIndex >= len(closeContour):
                    #             tobeAddPointIndex = 0
                    #         rightLength -= tobeAddLength
                    #         # ? spcanvas.js中第3172行是leftPos leftPos = tobeAddPos
                    #         rightPos = tobeAddPos
                    #         tobeAddPos = closeContour[tobeAddPointIndex]
                    #         # ? spcanvas.js 第 3174行也是leftPos
                    #         tobeAddLength = self.geoInstance.getDis(rightPos, tobeAddPos)

                    #     if rightLength < tobeAddLength:
                    #         # ? spcanvas.js 3177行也是leftPos
                    #         interPos = self.geoInstance.getPosinLine(rightPos, tobeAddPos, rightLength)
                    #         self.m_drawInterContour['dots2'].append(interPos)
                    #         curveBag['dots'].append([interPos[0] + diffVector[0], interPos[1] + diffVector[1]])
                    if rightLength != 0:
                        rightPos = closePos
                        tobeAddPointIndex = closePointIndex + 1
                        if tobeAddPointIndex >= len(closeContour):
                            print('cccc')
                            tobeAddPointIndex = 0
                        tobeAddPos = closeContour[tobeAddPointIndex]
                        # if tobeAddPos is None:
                        tobeAddLength = self.geoInstance.getDis(rightPos, tobeAddPos)
                        while tobeAddLength <= rightLength:
                            self.m_drawInterContour['dots2'].append(tobeAddPos)
                            curveBag['dots'].append([tobeAddPos[0] + diffVector[0], tobeAddPos[1] + diffVector[1]])
                            tobeAddPointIndex += 1
                            if tobeAddPointIndex >= len(closeContour):
                                print('right: pointIndex >= len(closeContour)')
                                tobeAddPointIndex = 0
                            rightLength -= tobeAddLength
                            # ? spcanvas.js中第3172行是leftPos leftPos = tobeAddPos
                            rightPos = tobeAddPos
                            tobeAddPos = closeContour[tobeAddPointIndex]
                            # ? spcanvas.js 第 3174行也是leftPos
                            tobeAddLength = self.geoInstance.getDis(rightPos, tobeAddPos)

                        if rightLength < tobeAddLength:
                            # ? spcanvas.js 3177行也是leftPos
                            interPos = self.geoInstance.getPosinLine(rightPos, tobeAddPos, rightLength)
                            self.m_drawInterContour['dots2'].append(interPos)
                            curveBag['dots'].append([interPos[0] + diffVector[0], interPos[1] + diffVector[1]])
                else:
                    curveBag['dots']  = []   
            
            mapIndexStrokeBag[i] = curveBag
        
        mapIndexCurve = self.adjustStrokesofClass(mapIndexStrokeBag)

        mapIsovalueIndex = {}
        for curKey in mapIsovalueIndexContour:
            contour = mapIsovalueIndexContour[curKey]
            curKeySplit = curKey.split('_')
            Isovalue = curKeySplit[0]
            index = curKeySplit[1]
            if Isovalue not in mapIsovalueIndex.keys():
                mapIsovalueIndex[Isovalue] = index
            elif int(mapIsovalueIndex[Isovalue] < index):
                mapIsovalueIndex[Isovalue] = index
        liRelatedContour = []
        for curKey in mapIsovalueIndex:
            key_temp = curKey + '_' + mapIsovalueIndex[curKey]
            liRelatedContour.append(mapIsovalueIndexContour[key_temp])
        return [mapIndexCurve, mapIndexStrokeBag, liRelatedContour, [pointToCentroid_min, pointToCentroid_max], [maxLength, minLength]]

    def computeDisMatrix(self, data):
        distanceOpeInstance = DistanceOperation()
        liClusterId = list(data.keys())

        for i in range(len(liClusterId)):
            curCluster1 = liClusterId[i]
            curDots1 = data[curCluster1]
            for p in range(len(curDots1)):
                curDot1 = curDots1[p]
                for j in range(i, len(liClusterId)):
                    curCluster2 = liClusterId[j]
                    curDots2 = data[curCluster2]
                    for q in range(len(curDots2)):
                        curDot2 = curDots2[q]
                        # curDistance = distanceOpeInstance.getDifClassDotDistance([curDot1['x'], curDot1['y']], [curDot2['x'], curDot2['y']])
                        curDistance = self.geoInstance.getDis([curDot1['x'], curDot1['y']], [curDot2['x'], curDot2['y']])
                        self.m_mapDisMatrix[str(curCluster1) + '_' + str(p) + '-' + str(curCluster2) + '_' + str(q)] = curDistance
                        self.m_mapDisMatrix[str(curCluster2) + '_' + str(q) + '-' + str(curCluster1) + '_' + str(p)] = curDistance
    
    def computeSilhouette(self, clusterInfo):
        # 在全局定义了
        # m_mapClassIDDotIndexSihouttle = {}  # 以class为key存放当前类的所有点的关于sihoutte的一个比值，(当前值-最小silhoutte值) / (最大silhoutte值-最小silhoutte值)
        # m_mapClassIDMaxMinSI = {}   # 以class为key存放当前类silhoutte的最大最小值，数组形式

        disMatrix = self.m_mapDisMatrix
        g_Max = 1e-6
        g_Min = 1e6

        for i in range(len(clusterInfo)):
            curClusterId1 = clusterInfo[i]['classId']

            liNormalSIIndex = []    
            liSilhoutteIndex = []   
            silhoutte_min = 1e6
            silhoutte_max = 1e-6

            curLiDots1 = clusterInfo[i]['dots']
            
            #计算当前class中每个点与该类中其他点的距离求均值放于avgA中，
            #计算当前class中每个点与其他类中所有点的距离求均值，将最小的?的均值放于avgB中
            # 根据上面的avgA和avgB求silhouette
            for p in range(len(curLiDots1)):
                avgA = 0
                #avgA为和同一类点求距离
                for q in range(len(curLiDots1)):
                    # if self.m_mapDisMatrix[str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId1) + '_' + str(q)] == 0:
                    #     print('hhhhhhhh')
                    if((str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId1) + '_' + str(q)) in self.m_mapDisMatrix):
                        avgA += self.m_mapDisMatrix[str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId1) + '_' + str(q)]
                    else:
                        print('avgA test None')
                        avgA += 5
                avgA /= len(curLiDots1)
                sihoutte = None
                avgB = 1e6
                for j in range(len(clusterInfo)):
                    if (i == j):
                        continue
                    curClusterId2 = clusterInfo[j]['classId']
                    curLiDots2 = clusterInfo[j]['dots']
                    tempAvgB = 0
                    for k in range(len(curLiDots2)):
                        if (curClusterId1 > curClusterId2):
                            if((str(curClusterId2) + '_' + str(k) + '-' + str(curClusterId1) + '_' + str(p)) in self.m_mapDisMatrix == False):
                                print('avgB test None')
                                tempAvgB += 10
                            else:
                                if self.m_mapDisMatrix[str(curClusterId2) + '_' + str(k) + '-' + str(curClusterId1) + '_' + str(p)] == 0:
                                    print('hhhhh2')
                                tempAvgB += self.m_mapDisMatrix[str(curClusterId2) + '_' + str(k) + '-' + str(curClusterId1) + '_' + str(p)]
                        else:
                            if((str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId2) + '_' + str(k)) in self.m_mapDisMatrix == False):
                                print('avgB test None')
                                tempAvgB += 10
                            else:
                                if self.m_mapDisMatrix[str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId2) + '_' + str(k)] == 0:
                                    print('hhhhh3')
                                tempAvgB += self.m_mapDisMatrix[str(curClusterId1) + '_' + str(p) + '-' + str(curClusterId2) + '_' + str(k)]
                    # ? 只用计算该店与各个类分别的距离再选择其中最小的？
                    tempAvgB /= len(curLiDots2)
                    if(avgB > tempAvgB):
                        avgB = tempAvgB
            
                if(avgA < avgB):
                    sihoutte = 1 - avgA/avgB
                elif(avgA == avgB):
                    sihoutte = 0
                else:
                    sihoutte = avgB/avgA - 1
        
                liSilhoutteIndex.append(sihoutte)
                if(silhoutte_max < sihoutte):
                    silhoutte_max = sihoutte;
                if(silhoutte_min > sihoutte):
                    silhoutte_min = sihoutte;
                if(g_Min > sihoutte):
                    g_Min = sihoutte;
                if(g_Max < sihoutte):
                    g_Max = sihoutte
            
            # liSihoutteIndex 当前class的各个点的silhoutte
            # liNormalSIIndex 存放的是当前类的各个点的关于sihoutte的一个比值，(当前值-最小silhoutte值) / (最大silhoutte值-最小silhoutte值)
            for sihoutteIndex in range(len(liSilhoutteIndex)):
                liNormalSIIndex.append((liSilhoutteIndex[sihoutteIndex] - silhoutte_min) / (silhoutte_max - silhoutte_min))
            

            self.m_mapClassIdDotIndexSihouttle[curClusterId1] = liNormalSIIndex
            self.m_mapClassIdMaxMinSI[curClusterId1] = [silhoutte_max, silhoutte_min]
    
    def computeDotStrokes(self, clusterInfo):
        m_WingType = 'outcontour'

        for i in range(len(clusterInfo)):
            curClusterId = clusterInfo[i]['classId']
            curDots = clusterInfo[i]['dots']

            liSihoutteIndex = []
            liSensitivity = []

            if(curClusterId in self.m_mapClassIdDotIndexSihouttle.keys()):
                liSihoutteIndex = self.m_mapClassIdDotIndexSihouttle[curClusterId]
            if(curClusterId in self.m_mapClassIdDotIndexSensitivity.keys()):
                liSensitivity = self.m_mapClassIdDotIndexSensitivity[curClusterId]

            liKDE = clusterInfo[i]['density']
            mapContours = clusterInfo[i]['contours']
            mapCounts = clusterInfo[i]['counts']
            mainContour = clusterInfo[i]['maincontour']
            mainIsovalue = clusterInfo[i]['mainIsovalue']
            self.m_test_mainContour[curClusterId] = mainContour

            mapMainIsoContour = {}
            mapDotIndexDeepIsoIndex = {}

            mapIsovalueCanvasContours = self.convertContourToCanvas(mapContours)
            #不知道排序的作用
            liSortedIsovalueStr = self.sortIsovalue(mapContours)
            clusterInfo[i]['sortedIsovalue'] = liSortedIsovalueStr

            outContour = []
            # spcanvas.js第1575行有几个if判断，不知道选哪个
            outIsovalue = mainIsovalue
            outContour = mainContour

            # 根据公共轮廓插值，并根据中心点与该点连线与contour的交集找到在contour上离该点最近的点
            # ? 如果 只用最外层的contour作为标准，那插值的作用
            mapNewIndexContours = self.interpolateContourSelf(outContour, 20)

            mapNewCanvasContours = {}   #存放的是最外层的最开始生成的公共contour
            mapNewIsovalueInnerContours = {}    #存放的是interpolateContourSelf生成的所有插值生成的contour
            mapNewCanvasContours[outIsovalue] = [outContour]
            mapNewIsovalueInnerContours[outIsovalue] = mapNewIndexContours

            # result = self.preLocate(curDots, liSihoutteIndex, liSensitivity, liKDE, 
            #     mapNewCanvasContours, mapNewIsovalueInnerContours, [outIsovalue])
            result = self.preLocate(curDots, liSihoutteIndex, liSensitivity, liKDE, 
                mapNewCanvasContours, mapNewIsovalueInnerContours, [outIsovalue])
            
            self.m_mapClassIdMaxMinLength[curClusterId] = [result[4][0], result[4][1]]
            clusterInfo[i]['canvascontours'] = []

            for iso in mapIsovalueCanvasContours:
                for temp in range(len(mapIsovalueCanvasContours[iso])):
                    clusterInfo[i]['canvascontours'].append(mapIsovalueCanvasContours[iso][temp])

            clusterInfo[i]['interpolatecontours'] = []
            for index_temp in mapNewIsovalueInnerContours[outIsovalue]:
                clusterInfo[i]['interpolatecontours'].append(mapNewIsovalueInnerContours[outIsovalue][index_temp])
            clusterInfo[i]['mainContour'] = mainContour

            self.m_mapClassIdDotIndexStroke[curClusterId] = result[0]
            self.m_mapClassIdDotIndexStrokeBag[curClusterId] = result[1]
            self.m_WC_ClassIDPointDis_minmax[curClusterId] = result[3]
        return clusterInfo

    def startDrawWinglets(self, dotsData, clusterInfo):
        liClusterInfo = clusterInfo['clusters']
        canvasRange = clusterInfo['canvasRange']
        self.m_xMin = canvasRange[0]
        self.m_xMax = canvasRange[1]
        self.m_yMin = canvasRange[2]
        self.m_yMax = canvasRange[3]

        self.computeDisMatrix(dotsData)
        self.computeSilhouette(liClusterInfo)

        curClusterInfo = self.computeDotStrokes(liClusterInfo)
        # print('self.m_mapClassIdDotIndexStroke', self.m_mapClassIdDotIndexStroke)
        # self.computeColorMap(liClusterInfo)
        return curClusterInfo, self.m_mapClassIdDotIndexStroke, self.m_test_mainContour








import math;
class getGeoInfo():
    g_strokeMaxLength = 100
    g_strokeMinLength = 0

    def getCentroid(self, liPos):
        centroid = [0, 0]
        if(len(liPos) < 0):
            return centroid
        for i in range(len(liPos)):
            centroid[0] += liPos[i][0]
            centroid[1] += liPos[i][1]
        centroid[0] /= len(liPos)
        centroid[1] /= len(liPos)
        return centroid
    
    def getDis(self, pos1, pos2):
        return math.sqrt((pos1[0] - pos2[0]) * (pos1[0] - pos2[0]) + (pos1[1] - pos2[1]) * (pos1[1] - pos2[1]))
    
    def getPosinLine(self, pos1, pos2, curLen):
        lineDis = self.getDis(pos1, pos2)
        delta = curLen / lineDis
        return [pos1[0] + (pos2[0] - pos1[0]) * delta, pos1[1] + (pos2[1] - pos1[1]) * delta]
    
    def getLengthbyContourIndex(self, curIsoIndex):
        curMaxIsoIndex = 20
        curMinIsoIndex = 0
        # ? 定义在geooperate.js第279行
        curMaxLength = self.g_strokeMaxLength
        curMinLength = self.g_strokeMinLength

        return curMinLength + (curMaxLength - curMinLength) * (curMaxIsoIndex - curIsoIndex) / (curMaxIsoIndex - curMinIsoIndex)

    def getLineXYatPercent(self, startPt, endPt, percent):
        # if(startPt.x):
        dx = endPt['x'] - startPt['x']
        dy = endPt['y'] - startPt['y']
        X = startPt['x'] + dx*percent
        Y = startPt['y'] + dy*percent

        return {'x': X, 'y': Y}
     



import tkinter as tk
window = tk.Tk()            #主窗口
window.title('my window')   #窗口标题
window.geometry('200x100')  #窗口尺寸
 

# import handler.drawhandler
# import handler.handler_all
# import handler.buttonhandler
# import handler.geoOperation
# import handler.kdehandler
# import handler.mongohandler
# import handler.wingletstephandler
from handler.drawhandler import DrawAllHandler
print('end')

### 这里是窗口的内容###
 
window.mainloop()           #循环消息，让窗口活起来




# import tkinter;
# import tkinter.messagebox;

# from tkinter import *;

# import tkinter as tt 
# import tkinter.messagebox

# from tkinter import *

# import numpy as np
# import pandas as pd
# from threading import Timer;

# import math;

# from .buttonhandler import ButtonOperation


# buttonOpeInstance = ButtonOperation()
# import matplotlib
# matplotlib.use("TkAgg")
import tkinter as tk
window = tk.Tk()
# window.mainloop()
# root = tt.Tk()
# root.geometry('1300x1300')
# # root.minsize(width=400, height=200)
# root.resizable(1, 1)

# root.config(bg="white")
# root.attributes('alpha', 0.65)
# root.attributes('-topmost', 1)


# cv = tt.Canvas(root, width=1000, height=1000, bg='white', scrollregion=(-50, -50, 950, 950))
print('ttttt')




class ColorMap():
    colorDict = {'1': 'red', '2': 'blue', '3':'pink', '4': 'orange', '5':'purple', '6':'indigo', '7': 'brown', '8': 'green', '9': 'yellow'}
    colorList = ['red', 'blue', 'pink', 'orange', 'purple', 'indigo']

globalColorHandler = ColorMap()

class DrawCircle():
    originCircleInfoArr = []
    colorRadius = 3.5
    commonFateData = []
    commonFateTimerArr = []
    commonFateGlobalInterval = 5
    globalMaxDensityPointsArr = []

    def drawProximityCircle(self, proximityPointsArr):
        print('drawCircle')
        # print('dots', dots)
        # self.globalMaxDensityPointsArr = globalMaxDensityPoints
        for i in range(len(proximityPointsArr)):
            curClassIdDots = proximityPointsArr[i]['curClassProximityPoints']
            curClassId = proximityPointsArr[i]['classId']

            for j in range(len(curClassIdDots)):
                curClassIdCurDot = curClassIdDots[j]
                # 用tkinter画圆形
                curCircle = cv.create_oval(curClassIdCurDot[0]-self.colorRadius, curClassIdCurDot[1]-self.colorRadius, curClassIdCurDot[0]+self.colorRadius, curClassIdCurDot[1]+self.colorRadius, tags="proximityDot",fill=globalColorHandler.colorDict[curClassId])
    
    def drawCircleTest(self, clusterInfo, globalMaxDensityPoints):
        print('drawCircle')
        # print('dots', dots)
        self.globalMaxDensityPointsArr = globalMaxDensityPoints
        for i in range(len(clusterInfo)):
            curClassCircleArr = []
            curClassCircleDict = {}

            curClassIdDots = clusterInfo[i]['transferDots']
            curClassId = clusterInfo[i]['classId']
            
            curClassCircleDict['classId'] = curClassId

            for j in range(len(curClassIdDots)):
                curClassIdCurDot = curClassIdDots[j]
                # 用tkinter画圆形
                curCircle = cv.create_oval(curClassIdCurDot[0]-self.colorRadius, curClassIdCurDot[1]-self.colorRadius, curClassIdCurDot[0]+self.colorRadius, curClassIdCurDot[1]+self.colorRadius, tags="originDot",fill=globalColorHandler.colorDict[curClassId])
                curClassCircleArr.append(curCircle)
            curClassCircleDict['originCircle'] = curClassCircleArr
            self.originCircleInfoArr.append(curClassCircleDict)

    def commonFateControlFunc(self):
        print('test')
        for i in range(len(self.commonFateData)):
            curClassId = self.commonFateData[i]['classId']
            curDot = self.commonFateData[i]['commonFateDot']
            
            curTimer = self.commonFateTimerArr[i]

            

    def commonFateTimerFunc(self, curDataIndex, curCount):
        curCommonFateData = self.commonFateData[curDataIndex]['commonFateDot']
        isPlus = True
        
        if curCount > 4:
            isPlus = False
        elif curCount <= 1:
            isPlus = True
        
        # curTimer = Timer(5, self.commonFateTimerFunc, (, curCount))

        if isPlus:
            curCount += 1
        else:
            curCount -= 1
        
        # curTimer.start()

    def drawCommonFateCircleByPointAlone(self, clusterInfo):
        print('common fate effect')
        print('globalMaxDensityPoints', self.globalMaxDensityPointsArr)

        cycleCount = 60
        # cycleCount = 30
        moveMaxDistance = 50
        everyDistance = moveMaxDistance / cycleCount

        for i in range(len(clusterInfo)):
            curCommonFateDict = {}
            
            curTimer = Timer(5, self.commonFateTimerFunc, (i, 1))
            self.commonFateTimerArr.append(curTimer)

            curClassIdDots = clusterInfo[i]['transferDots']
            curClassId = clusterInfo[i]['classId']
            curCentroid = clusterInfo[i]['centroid']
            curMaxDensityPoints = clusterInfo[i]['maxDensityPoints']

            centroidCircleRadius = self.colorRadius + 3
            # cv.create_oval(curCentroid[0]-centroidCircleRadius, curCentroid[1]-centroidCircleRadius, curCentroid[0]+centroidCircleRadius, curCentroid[1]+centroidCircleRadius, tags="centroid",fill='black')

            curCommonFateDict['classId'] = clusterInfo[i]['classId']
            curCommonFateDict['commonFateDot'] = []
            curCommonFateDict['curVector'] = []

            # curMaxDensityPoint = curMaxDensityPoints[math.floor(len(curMaxDensityPoints) / 2)]
            curMaxDensityPoint = self.globalMaxDensityPointsArr[i]
            
            curVector = [(curMaxDensityPoint[0] - curCentroid[0]), (curMaxDensityPoint[1] - curCentroid[1])]
            # curVector = [(curCentroid[0] - curClassIdDots[j][0]), (curCentroid[1] - curClassIdDots[j][0])]
            
            # curCommonFateDot = [[curClassIdDots[j][0], curClassIdDots[j][1]], [curVector[0] + curClassIdDots[j][0], curVector[1] + curClassIdDots[j][1]]]
            # for k in range(4):
            #     curCommonFateDot.append([curVector[0] / (4-j), curVector[1] / (4-j)])
            if abs(curVector[0]) > abs(curVector[1]):
                base = curVector[0] / moveMaxDistance
                curVector[0] /= base
                curVector[1] /= base
            else:
                base = curVector[1] / moveMaxDistance
                curVector[0] /= base
                curVector[1] /= base
            
            self.commonFateData.append(curVector)

            # 数据举例
            # [{  'classId':1, 
            #     'commonFateDot':[
            #                         [
            #                             [dot1Interpolate1X,dot1Interpolate1Y], 
            #                             [dot1Interpolate2X,dot1Interpolate2Y], 
            #                             [dot1Interpolate3X,dot1Interpolate3Y]
            #                         ],
            #                         [
            #                             [dot2Interpolate1X,dot2Interpolate1Y], 
            #                             [dot2Interpolate2X,dot2Interpolate2Y], 
            #                             [dot2Interpolate3X,dot2Interpolate3Y]
            #                         ],
            #                         ...
            #                     ]
            #  }, 
            #  {'classId':5, 'commonFateDot': []},
            #  ...
            # ]
            # for j in range(len(curClassIdDots)):
            #     curCommonFateDot = []
                # curVector = [(curClassIdDots[j][0] - curCentroid[0]), (curClassIdDots[j][1] - curCentroid[1])]
                # # curVector = [(curCentroid[0] - curClassIdDots[j][0]), (curCentroid[1] - curClassIdDots[j][0])]
                
                # # curCommonFateDot = [[curClassIdDots[j][0], curClassIdDots[j][1]], [curVector[0] + curClassIdDots[j][0], curVector[1] + curClassIdDots[j][1]]]
                # # for k in range(4):
                # #     curCommonFateDot.append([curVector[0] / (4-j), curVector[1] / (4-j)])
                # if abs(curVector[0]) > abs(curVector[1]):
                #     base = curVector[0] / moveMaxDistance
                #     curVector[0] /= base
                #     curVector[1] /= base
                # else:
                #     base = curVector[1] / moveMaxDistance
                #     curVector[0] /= base
                #     curVector[1] /= base
            
                # for k in range(cycleCount):
                #     # if i == 0 and j == 0 and k == 10:
                #     #     print('curClassIdDots[j][0] - curCentroid[0]', curClassIdDots[j][0] - curCentroid[0])
                #     #     print('curVector[0]', curVector[0])
                #     #     print('test', curVector[0] / cycleCount * (k+1)*everyDistance)
                #     curCommonFateDot.append([curClassIdDots[j][0] + (k+1) * curVector[0] / cycleCount, curClassIdDots[j][1] + (k+1) * curVector[1] / cycleCount])
                # curCommonFateDict['commonFateDot'].append(curCommonFateDot)
                # curCommonFateDict['curVector'].append(curVector)
            # self.commonFateData.append(curCommonFateDict)
         
        for i in range(cycleCount):
            for j in range(len(self.commonFateData)):
                # curCommonFateClassId = self.commonFateData[j]['classId']
                # curCommonFateDots = self.commonFateData[j]['commonFateDot']
                curCommonFateDotVector = self.commonFateData[j]
                curOriginCircle = self.originCircleInfoArr[j]['originCircle']


                for k in range(len(curOriginCircle)):
                    # curCommonFateDotVector = self.commonFateData[j]['curVector'][k]
                    cv.move(curOriginCircle[k], curCommonFateDotVector[0]/cycleCount, curCommonFateDotVector[1]/cycleCount)
                    # if i == 0:
                    #     # ?有点问题,暂时以continue跳过这个过程
                    #     continue
                    #     # print('curOriginCircle[k]', curOriginCircle[k])
                    #     # print('curCommonFateDots[k]', curCommonFateDots[k])
                    #     print("clusterInfo[j]['transferDots'][k]", clusterInfo[j]['transferDots'][k])
                    #     cv.move(curOriginCircle[k], curCommonFateDots[k][i][0]-clusterInfo[j]['transferDots'][k][0], curCommonFateDots[k][i][1]-clusterInfo[j]['transferDots'][k][1])
                    # else:
                    #     # print('curCommonFateDots[k][0]', curCommonFateDots[k][0])
                    #     cv.move(curOriginCircle[k], curCommonFateDots[k][i][0] - curCommonFateDots[k][i-1][0], curCommonFateDots[k][i][1] - curCommonFateDots[k][i-1][1])
            cv.update()
            # time.sleep(0.05)
            time.sleep(0.025)
        # self.commonFateControlFunc()

    

    def drawCommonFateCircle(self, clusterInfo):
        print('common fate effect')
        print('globalMaxDensityPoints', self.globalMaxDensityPointsArr)

        cycleCount = 60
        # cycleCount = 30
        moveMaxDistance = 50
        everyDistance = moveMaxDistance / cycleCount

        for i in range(len(clusterInfo)):
            curCommonFateDict = {}
            
            curTimer = Timer(5, self.commonFateTimerFunc, (i, 1))
            self.commonFateTimerArr.append(curTimer)

            curClassIdDots = clusterInfo[i]['transferDots']
            curClassId = clusterInfo[i]['classId']
            curCentroid = clusterInfo[i]['centroid']
            curMaxDensityPoints = clusterInfo[i]['maxDensityPoints']

            centroidCircleRadius = self.colorRadius + 3
            # cv.create_oval(curCentroid[0]-centroidCircleRadius, curCentroid[1]-centroidCircleRadius, curCentroid[0]+centroidCircleRadius, curCentroid[1]+centroidCircleRadius, tags="centroid",fill='black')

            curCommonFateDict['classId'] = clusterInfo[i]['classId']
            curCommonFateDict['commonFateDot'] = []
            curCommonFateDict['curVector'] = []

            # curMaxDensityPoint = curMaxDensityPoints[math.floor(len(curMaxDensityPoints) / 2)]
            curMaxDensityPoint = self.globalMaxDensityPointsArr[i]
            
            curVector = [(curMaxDensityPoint[0] - curCentroid[0]), (curMaxDensityPoint[1] - curCentroid[1])]
            # curVector = [(curCentroid[0] - curClassIdDots[j][0]), (curCentroid[1] - curClassIdDots[j][0])]
            
            # curCommonFateDot = [[curClassIdDots[j][0], curClassIdDots[j][1]], [curVector[0] + curClassIdDots[j][0], curVector[1] + curClassIdDots[j][1]]]
            # for k in range(4):
            #     curCommonFateDot.append([curVector[0] / (4-j), curVector[1] / (4-j)])
            if abs(curVector[0]) > abs(curVector[1]):
                base = curVector[0] / moveMaxDistance
                curVector[0] /= base
                curVector[1] /= base
            else:
                base = curVector[1] / moveMaxDistance
                curVector[0] /= base
                curVector[1] /= base
            
            self.commonFateData.append(curVector)

            # 数据举例
            # [{  'classId':1, 
            #     'commonFateDot':[
            #                         [
            #                             [dot1Interpolate1X,dot1Interpolate1Y], 
            #                             [dot1Interpolate2X,dot1Interpolate2Y], 
            #                             [dot1Interpolate3X,dot1Interpolate3Y]
            #                         ],
            #                         [
            #                             [dot2Interpolate1X,dot2Interpolate1Y], 
            #                             [dot2Interpolate2X,dot2Interpolate2Y], 
            #                             [dot2Interpolate3X,dot2Interpolate3Y]
            #                         ],
            #                         ...
            #                     ]
            #  }, 
            #  {'classId':5, 'commonFateDot': []},
            #  ...
            # ]
            # for j in range(len(curClassIdDots)):
            #     curCommonFateDot = []
                # curVector = [(curClassIdDots[j][0] - curCentroid[0]), (curClassIdDots[j][1] - curCentroid[1])]
                # # curVector = [(curCentroid[0] - curClassIdDots[j][0]), (curCentroid[1] - curClassIdDots[j][0])]
                
                # # curCommonFateDot = [[curClassIdDots[j][0], curClassIdDots[j][1]], [curVector[0] + curClassIdDots[j][0], curVector[1] + curClassIdDots[j][1]]]
                # # for k in range(4):
                # #     curCommonFateDot.append([curVector[0] / (4-j), curVector[1] / (4-j)])
                # if abs(curVector[0]) > abs(curVector[1]):
                #     base = curVector[0] / moveMaxDistance
                #     curVector[0] /= base
                #     curVector[1] /= base
                # else:
                #     base = curVector[1] / moveMaxDistance
                #     curVector[0] /= base
                #     curVector[1] /= base
            
                # for k in range(cycleCount):
                #     # if i == 0 and j == 0 and k == 10:
                #     #     print('curClassIdDots[j][0] - curCentroid[0]', curClassIdDots[j][0] - curCentroid[0])
                #     #     print('curVector[0]', curVector[0])
                #     #     print('test', curVector[0] / cycleCount * (k+1)*everyDistance)
                #     curCommonFateDot.append([curClassIdDots[j][0] + (k+1) * curVector[0] / cycleCount, curClassIdDots[j][1] + (k+1) * curVector[1] / cycleCount])
                # curCommonFateDict['commonFateDot'].append(curCommonFateDot)
                # curCommonFateDict['curVector'].append(curVector)
            # self.commonFateData.append(curCommonFateDict)
         
        for i in range(cycleCount):
            for j in range(len(self.commonFateData)):
                # curCommonFateClassId = self.commonFateData[j]['classId']
                # curCommonFateDots = self.commonFateData[j]['commonFateDot']
                curCommonFateDotVector = self.commonFateData[j]
                curOriginCircle = self.originCircleInfoArr[j]['originCircle']
                

                for k in range(len(curOriginCircle)):
                    # curCommonFateDotVector = self.commonFateData[j]['curVector'][k]
                    cv.move(curOriginCircle[k], curCommonFateDotVector[0]/cycleCount, curCommonFateDotVector[1]/cycleCount)
                    # if i == 0:
                    #     # ?有点问题,暂时以continue跳过这个过程
                    #     continue
                    #     # print('curOriginCircle[k]', curOriginCircle[k])
                    #     # print('curCommonFateDots[k]', curCommonFateDots[k])
                    #     print("clusterInfo[j]['transferDots'][k]", clusterInfo[j]['transferDots'][k])
                    #     cv.move(curOriginCircle[k], curCommonFateDots[k][i][0]-clusterInfo[j]['transferDots'][k][0], curCommonFateDots[k][i][1]-clusterInfo[j]['transferDots'][k][1])
                    # else:
                    #     # print('curCommonFateDots[k][0]', curCommonFateDots[k][0])
                    #     cv.move(curOriginCircle[k], curCommonFateDots[k][i][0] - curCommonFateDots[k][i-1][0], curCommonFateDots[k][i][1] - curCommonFateDots[k][i-1][1])
            cv.update()
            # time.sleep(0.05)
            time.sleep(0.025)
        # self.commonFateControlFunc()


class DrawKDE():
    def drawKDEMap(self, clusterInfo):
        print('drawKDE')
        # liClusterInfo = clusterInfo['clusters']
        # for i in range(len(liClusterInfo)):
        #     # curDensityX = liClusterInfo[i]['m12'][0]
        #     # curDensityY = liClusterInfo[i]['m12'][1]
        #     # sns.kdeplot(curDensityX, curDensityY, shade=True,color='r')
        #     fig, ax = plt.subplots(num=100)
        #     curDensity = liClusterInfo[i]['density']
        #     im = ax.imshow(np.rot90(curDensity), cmap='Blues', extent=[0, 800, 0, 800])
        #     plt.show()

class DrawMainContour():
    def drawMainContour(self, clusterInfo):
        print('draw maincontour')
        for i in range(len(clusterInfo)):
            curMainContour = clusterInfo[i]['maincontour']
            curClassId = clusterInfo[i]['classId']
            curMainContourDotArr = []
            for p in range(len(curMainContour)):
                curMainContourDotArr.append(curMainContour[p][0])
                curMainContourDotArr.append(curMainContour[p][1])
            mainContourLine = cv.create_line(curMainContourDotArr, fill='orange', width=3, state="hidden", tags="mainContourLine")

    def drawTwoPointLine(self, clusterInfo, mapClassIdDotIndexStroke):
        colorRadius = 4
        for i in range(len(clusterInfo)):
            curClassId = clusterInfo[i]['classId']
            liDots = clusterInfo[i]['dots']
            mapIndexStrokeInfo = mapClassIdDotIndexStroke[curClassId]
            
            for j in range(len(liDots)):
                curOriginPos = liDots[j]
                curIntersectionPos = mapIndexStrokeInfo[j]['intersection']
                # curIntersectionPos = mapClassIdDotIndexStroke[curClassId]['intersection']
                # curOriginPos = mapClassIdDotIndexStroke[curClassId]['originDot']
                curLineArr = [curOriginPos, curIntersectionPos]
                line = cv.create_line(curLineArr, fill='green', width=3, state="hidden", tags="twoPointLine")
                circle = cv.create_oval(curIntersectionPos[0]-colorRadius, curIntersectionPos[1]-colorRadius, curIntersectionPos[0]+colorRadius, curIntersectionPos[1]+colorRadius, state="hidden", fill='green', tags="intersectionPos")


class DrawContour():
    print('drawContour')
    def drawContour(self, clusterInfo):
        for i in range(len(clusterInfo)):
            curContour = clusterInfo[i]['contours']
            curClassId = clusterInfo[i]['classId']
            curContourDotArr = []
            for isovalue in curContour:
                for p in range(len(curContour[isovalue][0])):
                    curContourDotArr.append(curContour[isovalue][0][p][0])
                    curContourDotArr.append(curContour[isovalue][0][p][1])
            contourLine = cv.create_line(curContourDotArr, fill=globalColorHandler.colorDict[curClassId], state="hidden", tags="contourLine")

class DrawWinglets():
    print('drawWinglets')

    def generateWings(self, clusterInfo, mapClassIdDotIndexStroke):
        for i in range(len(clusterInfo)):
            curClassId = clusterInfo[i]['classId']
            liDots = clusterInfo[i]['dots']
            liContour_interpolate = clusterInfo[i]['interpolatecontours']
            mapIndexCurve = mapClassIdDotIndexStroke[curClassId]
            classColor = 'black'
            self.addDotCurves_svg(curClassId, liDots, mapIndexCurve, classColor)

    def addDotCurves_svg(self, curClassId, dots, mapIndexCurve, strokeColr):
        for i in range(len(dots)):
            curCurve = mapIndexCurve[i]['curve']
            curCurveDotArr = []
            for j in range(len(curCurve)):
                curCurveDotArr.append(curCurve[j][0])
                curCurveDotArr.append(curCurve[j][1])
            line = cv.create_line(curCurveDotArr, fill=globalColorHandler.colorDict[curClassId], width=1.8, tags="wingletsLine")
        print('Draw Winglets Done')

class DrawAllHandler():
    # commonFateButton = None
    # intersectionPosButton = None
    # mainContourButton = None
    # twoPointLineButton = None
    # contourButton = None 
    # wingletsButton = None 
    # testClusterInfo = []
    # testCircleHander = None
    # maxDensityPointsArr = []
    # proximityPointsArr = []
    # isCommonFateStart = False
    # notCommonFateAndProximity = True
    # isProximityStart = False
    # clickIntervalOk = True

    # def init(self):
    #     self.testCircleHander = drawCircleHandler = DrawCircle()
    #     # self.initButton()
    #     drawKDEHandler = DrawKDE()
    #     drawMainContourHandler = DrawMainContour()
    #     drawContourHandler = DrawContour()
    #     drawWingletsHandler = DrawWinglets()
    #     return drawCircleHandler, drawKDEHandler, drawMainContourHandler, drawContourHandler, drawWingletsHandler

    # def initButton(self):
    #     self.proximityButton = tt.Button(root, text="proximity", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: self.startProximity())
    #     self.commonFateButton = tt.Button(root, text="commonFate", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: self.startCommonFate())
    #     self.intersectionPosButton = tt.Button(root, text="intersectionPos", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: buttonOpeInstance.orihiddenElement(cv, 'intersectionPos'))
    #     self.mainContourButton = tt.Button(root, text="mainContour", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: buttonOpeInstance.orihiddenElement(cv, 'mainContourLine'))
    #     self.twoPointLineButton = tt.Button(root, text="twoPointLine", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: buttonOpeInstance.orihiddenElement(cv, 'twoPointLine'))
    #     self.contourButton = tt.Button(root, text="Contour", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: buttonOpeInstance.orihiddenElement(cv, 'contourLine'))
    #     self.wingletsButton = tt.Button(root, text="Winglets", bg='white', activebackground='#F0F0F0', bd=1, relief="ridge", command = lambda: buttonOpeInstance.orihiddenElement(cv, 'wingletsLine'))

    def getInfo(self, clusterInfo, maxDensityPointsArr, proximityPointsArr):
        self.testClusterInfo = clusterInfo
        self.maxDensityPointsArr = maxDensityPointsArr
        self.proximityPointsArr = proximityPointsArr
    
    # def startCommonFate(self):
    #     if not self.clickIntervalOk:
    #         print('click interval not ok, program is running, please wait')
    #         return 0
    #     # ? 先点proximity再点commonFate,commonFate函数运行但是没动画
    #     if not self.isCommonFateStart and self.notCommonFateAndProximity:
    #         print('commonFate state', self.isCommonFateStart)
    #         self.clickIntervalOk = False
    #         self.testCircleHander.drawCommonFateCircle(self.testClusterInfo)
    #         self.isCommonFateStart = True
    #         self.notCommonFateAndProximity = False
    #         self.isProximityStart = False
    #         time.sleep(2)
    #         self.clickIntervalOk = True
    #         print('clickIntervalOk')
    #     elif not self.notCommonFateAndProximity and self.isCommonFateStart:
    #         print('commonFate state', self.isCommonFateStart)
    #         self.clickIntervalOk = False
    #         cv.delete('originDot')
    #         cv.delete('proximityDot')
    #         self.testCircleHander.drawCircleTest(self.testClusterInfo, self.maxDensityPointsArr)
    #         self.isCommonFateStart = False
    #         self.notCommonFateAndProximity = True
    #         self.isProximityStart = False
    #         time.sleep(2)
    #         self.clickIntervalOk = True
    #         print('clickIntervalOk')
        
    # def startProximity(self):
    #     if not self.clickIntervalOk:
    #         print('click interval not ok, program is running, please wait')
    #         return 0

    #     if not self.notCommonFateAndProximity and self.isProximityStart:
    #         print('proximity state', self.isProximityStart)
    #         self.clickIntervalOk = False
    #         cv.delete('proximityDot')
    #         self.testCircleHander.drawCircleTest(self.testClusterInfo, self.maxDensityPointsArr)
    #         self.isCommonFateStart = False
    #         self.notCommonFateAndProximity = True
    #         self.isProximityStart = False
    #         time.sleep(2)
    #         self.clickIntervalOk = True
    #         print('clickIntervalOk')
    #     elif not self.isProximityStart and self.notCommonFateAndProximity:
    #         print('commonFate state', self.isCommonFateStart)
    #         self.clickIntervalOk = False
    #         cv.delete('originDot')
    #         self.testCircleHander.drawProximityCircle(self.proximityPointsArr)
    #         self.isCommonFateStart = False
    #         self.notCommonFateAndProximity = False 
    #         self.isProximityStart = True
    #         time.sleep(2)
    #         self.clickIntervalOk = True
    #         print('clickIntervalOk')

    # def endDraw(self):
    #     # self.contourButton.pack(side=RIGHT)
    #     # self.wingletsButton.pack(side=RIGHT)
    #     self.proximityButton.place(x=1025, y=350, width=100)
    #     self.commonFateButton.place(x=1025, y=400, width=100)
    #     self.intersectionPosButton.place(x=1150, y=400, width=100)
    #     self.mainContourButton.place(x=1025, y=450, width=100)
    #     self.twoPointLineButton.place(x=1150, y=450, width=100)
    #     self.contourButton.place(x=1025, y=500, width=70)
    #     self.wingletsButton.place(x=1110, y=500, width=70)
    #     cv.place(x=0, y=0)
    #     root.mainloop()








from .handler.handler_all import *
# from handler import handler_all as AllHandler
# from .handler.handler_all import OperationHandler
# from handler.handler_all import OperationHandler
# import handler.handler_all as AllHandler

# setting = dict(
#     static_path=os.path.join(os.path.dirname(__file__), './'),
#     template_path=os.path.join(os.path.dirname(__file__), './'),

# )

# url = []

# application = tornado.web.Application(
#     handlers=url,
#     debug=True,
#     **setting
# )

# serverPort = 30001
# define("port", default=serverPort, help="run on the given port", type=int)


# def main():
#     tornado.options.parse_command_line()
#     http_server = tornado.httpserver.HTTPServer(application)
#     print('Development server is running at http://127.0.0.1:%s/' % options.port)
#     print('Quit the server with Control-C')
#     tornado.ioloop.IOLoop.instance().start()

# if __name__ == '__main__':
#     testName = 'dots'
#     outputDots(testName)
#     main()

def draw(data, dataInputType='normal Array'):
    # outputDots(testName)
    operationInstance = OperationHandler()
    operationInstance.drawWinglets(data)
    if dataInputType == 'mongodbData':
        print('input type', dataInputType)
    else:
        print('input type', dataInputType)
    operationInstance.endDraw()
    # main()

import json
dataDict = {}
f = open('./testFile.json', 'r')
dataDict = json.loads(f.read())
draw(dataDict)




import math;
import numpy as np;
from scipy import stats;
from skimage import measure;
from sklearn.cluster import KMeans

from .geoOperation import getGeoInfo;
from shapely.geometry import Point;
from shapely.geometry import Polygon;

import seaborn as sns
import numpy as np
import pandas as pd
# import matplotlib.pyplot as plt

class Distance():
    def distanceCompute(self, dotInfo):
        sourceAndTarget = [];
        for i in range(len(dotInfo)):
            minDistance = float('inf');  #创建无穷大的值
            minTarget = 0
            for j in range(len(dotInfo)):
                if (j == i):
                    continue;
                else:
                    dx = abs(dotInfo[i]['x'] - dotInfo[j]['x']);
                    dy = abs(dotInfo[i]['y'] - dotInfo[j]['y']);
                    distance = math.sqrt(dx*dx + dy*dy);
                    if (distance < minDistance):
                        minDistance = distance
                        mintarget = j
            sourceAndTarget.append({
                'source': i,
                'target': j,
                'mindistance': minDistance
            })
        # print('sourceAndTarget', sourceAndTarget)
        return sourceAndTarget

class KDE():
    def listZip(self, m1, m2):
        list1 = list(m1);
        list2 = list(m2);
        return list(zip(list1, list2));

    def dot2Canvas(self, dotxy):
        #? 为啥是100 - 
        # ? 为啥 dotxy的 0 1 元素换位了
        dot = [dotxy[1], 100 - dotxy[0]]
        # dot = [dotxy[1], dotxy[0]]
        scale = 8
        return [scale * dot[0], scale * dot[1]] 

    def convert2Canvas(self, contour):
        newContour = []
        for dot in contour:
            scaleDot = self.dot2Canvas(dot)
            newContour.append(scaleDot)
        return newContour
    
    def kdeCore(self, m1, m2, xmin, xmax, ymin, ymax):
        # X, Y = np.mgrid[xmin:xmax:800j, ymin:ymax:800j]
        X, Y = np.mgrid[xmin:xmax:100j, ymin:ymax:100j]
        positions = np.vstack([X.ravel(), Y.ravel()]);
        values = np.vstack([m1, m2]);
        kernel = stats.gaussian_kde(values)
        Z = np.reshape(kernel(positions).T, X.shape)
        # drawKDEZX, drawKDEZY = np.reshape(kernel(positions).T, (2, 5000))
        # p1 = sns.kdeplot(drawKDEZX, drawKDEZY, cmap="Blues", shade=True, shade_lowest=True, )

        # plt.show()
        return np.rot90(Z)
        # return Z

    def getGlobalMaxDensityPoint(self, globalZ, curClassDots):
        mapIsovalueContours = {}
        isoPosNum = 10
        baseValue = 1e-9
        liIsovalue = []
        minDensity = globalZ.ravel().min()
        maxDensity = globalZ.ravel().max()
        proximityLowDensityPoints = []

        for i in range(isoPosNum-1, 0, -1):
            thresholdDensity = baseValue + i * (maxDensity - baseValue)/isoPosNum
            thresholdContours = measure.find_contours(globalZ, thresholdDensity, fully_connected='high')
            thresholdPolygonArr = []
            maxDensityPoints = []
            thresholdContainPointCount = 0

            for n, contour in enumerate(thresholdContours):
                contour = contour.tolist()
                thresholdCanvasContour = self.convert2Canvas(contour)
                thresholdPolygon = Polygon(thresholdCanvasContour)
                thresholdPolygonArr.append(thresholdPolygon)
            for dot in curClassDots:
                    point = Point([dot['x'], dot['y']])
                    for index_temp in range(len(thresholdPolygonArr)):
                        polygon = thresholdPolygonArr[index_temp]
                        if(polygon.contains(point) == True):
                            thresholdContainPointCount += 1
                            maxDensityPoints.append([dot['x'], dot['y']])
                        else:
                            proximityLowDensityPoints.append([dot['x'], dot['y']])
            if thresholdContainPointCount < 30 and thresholdContainPointCount > 0:
                break
        print('count', thresholdContainPointCount)
        return maxDensityPoints, proximityLowDensityPoints

    def getContours(self, Z, dots):
        mapIsovalueContours = {}
        isoPosNum = 10
        baseValue = 1e-9
        liIsovalue = []
        minDensity = Z.ravel().min()
        maxDensity = Z.ravel().max()
        # ?不知道该不该加, 因为不加最低的isoValue为 1e-9, 比1e-1小,但是liIsovalue为 0.1, 1e-09 ,但是为什么加上后就可以 在计算kde是rotate90°并且画的contour没问题了
        # liIsovalue.append(1e-1)

        testDensity = baseValue + 9 * (maxDensity - baseValue)/isoPosNum
        testMaxContours = measure.find_contours(Z, testDensity, fully_connected='high')
        testMaxPolygonArr = []
        testMaxContainPointCount = 0
        maxDensityPoints = []
        for n, contour in enumerate(testMaxContours):
            contour = contour.tolist()
            testMaxCanvasContour = self.convert2Canvas(contour)
            testMaxPolygon = Polygon(testMaxCanvasContour)
            testMaxPolygonArr.append(testMaxPolygon)
        for dot in dots:
                point = Point([dot['x'], dot['y']])
                for index_temp in range(len(testMaxPolygonArr)):
                    polygon = testMaxPolygonArr[index_temp]
                    if(polygon.contains(point) == True):
                        testMaxContainPointCount += 1
                        maxDensityPoints.append([dot['x'], dot['y']])
        # print('testMaxContainPointCount', testMaxContainPointCount)


        #根据设定的baseValue和isoPosNum按密度分段，将范围的每个值存入liIsoValue
        for i in range(isoPosNum):
            liIsovalue.append(baseValue + i * (maxDensity - baseValue)/isoPosNum)
        
        # print('liIsovalue', liIsovalue)
        
        #遍历设定的密度范围的分段值
        for i in range(len(liIsovalue)):
            #按照当前的 isoValue 生成contours
            curContours = measure.find_contours(Z, liIsovalue[i], fully_connected='high')

            curIsovalue = liIsovalue[i]
            #存放转化后的contours以及增加的信息
            curLiContours = []
            curLiPolygon = []

            #遍历按照当前isoValue生成的contour,因为同一个值可能生成两个互补重叠的contour，参考等高线
            for n, contour in enumerate(curContours):
                contour = contour.tolist();
                if (len(contour) == 0):
                    continue

                #将当前的contour以及设定count为0，用来统计当前isoValue生成的contour里有多少个点
                # curLiContours.append({
                #     'contour': contour,
                #     'count': 0
                # });
                # 缩放，此处我注释了
                curCanvasContour = self.convert2Canvas(contour)
                curLiContours.append({
                    'contour': curCanvasContour,
                    # 'contour': contour,
                    'count': 0
                });
                #按照当前的contour生成polygon，后面用来统计当前contour生成的polygon中点的个数
                # curPolygon = Polygon(contour)
                curPolygon = Polygon(curCanvasContour)
                curLiPolygon.append(curPolygon)
            
            #遍历数据点，然后遍历前面根据isoValue生成的polygon，判断当前数据点是否再polygon里面，在得话count加一
            for dot in dots:
                point = Point([dot['x'], dot['y']])
                for index_temp in range(len(curLiPolygon)):
                    polygon = curLiPolygon[index_temp]
                    if(polygon.contains(point) == True):
                        curLiContours[index_temp]['count'] += 1
            
            if (len(curLiContours) != 0):
                # 将contour和count信息以对象形式返回，且key是初始生成的各个isoValue
                mapIsovalueContours[str(curIsovalue)] = curLiContours
        return mapIsovalueContours, maxDensityPoints
        

class KDEHandler():
    geoInstance = getGeoInfo()

    def getXY(self, dotInfo):
        liX = []
        liY = []
        for i in range(len(dotInfo)):
            liX.append(dotInfo[i]['x'])
            liY.append(dotInfo[i]['y'])
        return np.array(liX), np.array(liY)
    
    def turpleToList(self, curTurpleData):
        curListData = list(curTurpleData)
        for item in curListData:
            curListData[curListData.index(item)] = list(item)
        return curListData

    def getAllClassDot(self, data):
        liAllDots = []
        for classId, dotsXYData in data.items():
            for i in range(len(dotsXYData)):
                liAllDots.append(dotsXYData[i])
        return liAllDots
    
    def computeKDE(self, data):
        # print('test data', data)
        estimator = KMeans(n_clusters=3)#构造聚类器
        # result = estimator.fit_predict(data)
        DistanceInstance = Distance()
        KDEContour = KDE()
        liCluster = [];
        liModifiedDots = []
        distanceCollect = [];
        contourPar = [];

        #定义密度场计算范围
        xmin = 0;
        xmax = 800;
        ymin = 0;
        ymax = 800;

        allDotsXYData = self.getAllClassDot(data)
        globalM1, globalM2 = self.getXY(allDotsXYData)
        globalZ1 = KDEContour.kdeCore(globalM1, globalM2, xmin, xmax, ymin, ymax)
        globalMaxDensityPoints = []
        proximityPoints = []

        for classId, dotsXYData in data.items():
            curProximityPoints = []
            curMaxDensityPoints, curLowDensityPoints = KDEContour.getGlobalMaxDensityPoint(globalZ1, dotsXYData)
            # print('len', len(curMaxDensityPoints))
            # print('curMaxDensityPoints', curMaxDensityPoints)
            if len(curMaxDensityPoints) > 3:
                proximityCentoridPoints = estimator.fit(curMaxDensityPoints).cluster_centers_.tolist()
                # print('curLowDensityPoints',curLowDensityPoints)
                curProximityPoints = curLowDensityPoints + proximityCentoridPoints
                # print('centroid', estimator.fit(curMaxDensityPoints).cluster_centers_.tolist())
            else:
                curProximityPoints = curLowDensityPoints + curMaxDensityPoints
            globalMaxDensityPoints.append(curMaxDensityPoints[math.floor(len(curMaxDensityPoints) / 2)])
            # print('curProximityPoints',curProximityPoints)
            proximityPoints.append({'classId': classId, 'curClassProximityPoints': curProximityPoints})
        # globalTransferContour, globalMaxDensityPoints = KDEContour.getContours(globalZ1, allDotsXYData)

        #对每个classId计算kde
        for classId, dotsXYData in data.items():
            mainIsovalue = 1e6
            stopCompare = False

            curDistance = DistanceInstance.distanceCompute(dotsXYData)
            m1, m2 = self.getXY(dotsXYData)
            liDots = KDEContour.listZip(m1, m2);

            Z1 = KDEContour.kdeCore(m1, m2, xmin, xmax, ymin, ymax);

            curDensity = list(Z1.ravel())
            curTransferDot = self.turpleToList(zip(list(m1), list(m2)))
            liModifiedDots.append(curTransferDot)

            # 生成初始的contour，返回的是以isovalue为key的对象
            # 每个key对应以当前isovalue生成的一个或多个contour以及该contour里面包含的点的数量
            transferContour, maxDensityPoints = KDEContour.getContours(Z1, dotsXYData)

            #将有getContours函数中提取的isoValue存放在liIsoValue中并排序
            liIsovalue = []
            mainContour = {}
            mainIsovalue = 1e6
            stopCompare = False

            for Isovalue_str in transferContour.keys():
                liIsovalue.append(float(Isovalue_str))
            liIsovalue = sorted(liIsovalue)
            
            mapBezierContour = {}
            mapIsoContourCount = {}
            
            preCount = len(dotsXYData)
            preContour = []
            preIsovalue = -1e6
            isoCount = 0


            for Isovalue in liIsovalue:
                maxCount = -1e6
                maxIsovalue = -1e6
                maxContour = []
                liNewContour = []
                liNewCount = []

                isoCount += 1
                Isovalue = str(Isovalue)
                liContour = transferContour[Isovalue]

                # 在每个isoValue下找到当前isovalue生成的contour中count最多的contour，并且将其放入当前isovalue下的 liNewContour和liNewCount中
                for temp in range(len(liContour)):
                    tempContour = liContour[temp]['contour']
                    tempCount = liContour[temp]['count']
                    if (maxCount < tempCount):
                        maxCount = tempCount
                        maxContour = tempContour
                        maxIsovalue = Isovalue
                    
                    beginPos = tempContour[0]
                    endPos = tempContour[-1]

                    if(abs(beginPos[0] - endPos[0]) > 10 or abs(beginPos[1] - endPos[1]) > 10):
                        continue
                    liNewContour.append(tempContour)
                    liNewCount.append(tempCount)
                # preCount为当前类所以的点的数量，maxCount为当前isoValue生成的众多contour中点数量最多的contour的点数量
                # 只选择其中最大的一个contour的点数量是因为如果算两个contour的总和的话，没有骤减继续比较时应该选取哪个contour就成了一个问题
                # 进行了比较precount - maxCount > 点数量的87%说明已经发生了骤减，少了13%的点，可以选择外层的contour，
                # 如果没有的话，说明maxCount的点很多很多，选取这个contour当前isoValue生成的众多contour的代表也是可以的
                # print('maxContour', maxContour)
                # print('preCount', preCount)
                # print('maxCount', maxCount)
                if(stopCompare == False):
                    if((preCount - maxCount) > int(preCount * 0.13)):
                        mainContour = preContour
                        mainIsovalue = preIsovalue
                        stopCompare = True
                    else:
                        preContour = maxContour
                        preIsovalue = maxIsovalue

                if (len(liNewContour) > 0):
                    mapBezierContour[Isovalue] = liNewContour;
                    mapIsoContourCount[Isovalue] = liNewCount

            # print('maincontour', mainContour)
            curCentroid = self.geoInstance.getCentroid(mainContour)
        
            liCluster.append({
                'classId': classId,
                'dots': liDots,
                'centroid': curCentroid,
                'transferDots': curTransferDot,
                'density': curDensity,
                'minDensity': Z1.ravel().min(),
                'maxDensity': Z1.ravel().max(),
                'distance': curDistance,
                'contours': mapBezierContour,   
                'counts': mapIsoContourCount,
                'maincontour': mainContour,
                'mainIsovalue': mainIsovalue,
                'maxDensityPoints': maxDensityPoints
            })
        return liModifiedDots, {'clusters': liCluster, 'canvasRange': [xmin, xmax, ymin, ymax]}, globalMaxDensityPoints, proximityPoints




import pymongo
from pymongo import MongoClient

# dbIp = 'localhost'


class MONGODBHANDLER:
    def connectDB(self, dbName, dbIp, port):
        self._connection = MongoClient(dbIp, port)
        # print(self._connection.list_database_names())
        self._db = self._connection[dbName]
        collectionName = 'points'
        self._collection = self._db[collectionName]

    def getDots (self, fieldsName):
        return self._collection.find_one({'name': '4'})


# from handler.mongohandler import MONGODBHANDLER
# from handler.kdehandler import KDEHandler
# from handler.wingletstephandler import WingletsStepHandler
# from handler.drawhandler import DrawAllHandler
from .kdehandler import KDEHandler
from .wingletstephandler import WingletsStepHandler
from .drawhandler import DrawAllHandler

# class DataInputHandler():
#     def __init__(self, inputType):
#         self.dataInputType = inputType
#         myDB = MONGODBHANDLER()
#         myDB.connectDB('First', 'localhost', 27017)

class OperationHandler():
    def __init__(self):
        # self.curData = DataInputHandler()
        self.kdeHandler = KDEHandler()
        self.wingletsStepHandler = WingletsStepHandler()
        self.drawHandler = DrawAllHandler()
        self.drawCircleHandler, self.drawKDEHandler, self.drawMainContourHandler, self.drawContourHandler, self.drawWingletsHandler = self.drawHandler.init()
        
    def drawWinglets(self, data):
        # dots = myDB.getDots(fieldsName)['dots']
        modifiedDots, clusterInfo, globalMaxDensityPoints, proximityPoints = self.kdeHandler.computeKDE(data)
        self.drawHandler.getInfo(clusterInfo['clusters'], globalMaxDensityPoints, proximityPoints)
        self.drawCircleHandler.drawCircleTest(clusterInfo['clusters'], globalMaxDensityPoints)
        # drawCircleHandler.drawProximityCircle(proximityPoints)
        # drawCircleHandler.drawCommonFateCircle(clusterInfo['clusters'])
        # drawKDEHandler.drawKDEMap(clusterInfo['clusters'])
        self.drawContourHandler.drawContour(clusterInfo['clusters'])
        self.drawMainContourHandler.drawMainContour(clusterInfo['clusters'])
        # print('*******')
        curClusterInfo, mapClassIdDotIndexStroke, liMainContour = self.wingletsStepHandler.startDrawWinglets(data, clusterInfo)
        self.drawMainContourHandler.drawTwoPointLine(curClusterInfo, mapClassIdDotIndexStroke)
        self.drawWingletsHandler.generateWings(curClusterInfo, mapClassIdDotIndexStroke)
    
    def endDraw(self):
        self.drawHandler.endDraw()
    # def drawWinglets(self):


 




class ButtonOperation(): 
    isCommonFateDisplay = False
    isIntersectionPosDisplay = False
    isMainContourDisplay = False
    isTwoPointLineState = False
    isContourDisplay = False
    isWingletsDisplay = True

    commonFateState = None
    intersectionPosState = None
    mainContourState = None
    twoPointLineState = None
    contourState = None
    wingletsState = None
    

    def orihiddenElement(self, curCanvas, tagName):
        print('curButtonTag', tagName)

        curControlState = None
        curControlDisplay = None

        if tagName == 'commonFate':
            self.isCommonFateDisplay = not self.isCommonFateDisplay
            if self.isCommonFateDisplay:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        elif tagName == 'intersectionPos':
            self.isIntersectionPosDisplay = not self.isIntersectionPosDisplay
            if self.isIntersectionPosDisplay:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        elif tagName == 'mainContourLine':
            self.isMainContourDisplay = not self.isMainContourDisplay
            if self.isMainContourDisplay:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        elif tagName == 'twoPointLine':
            self.isTwoPointLineState = not self.isTwoPointLineState
            if self.isTwoPointLineState:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        elif tagName == 'contourLine':
            self.isContourDisplay = not self.isContourDisplay
            if self.isContourDisplay:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        elif tagName == 'wingletsLine':
            self.isWingletsDisplay = not self.isWingletsDisplay
            if self.isWingletsDisplay:
                curControlState = 'normal'
            else:
                curControlState = 'hidden'
        
        curCanvas.itemconfig(tagName, state = curControlState)

    def mainContourButtonOpe(self, curCanvas):
        print('main contour button click')
        mainContourState = None
        self.isMainContourDisplay = not self.isMainContourDisplay
        # curCanvas.delete('contourLine')
        if self.isMainContourDisplay:
            mainContourState = 'normal'
        else: 
            mainContourState = 'hidden'
        curCanvas.itemconfig('mainContourLine', state = mainContourState)

    def contourButtonOpe(self, curCanvas):
        print('contour button click')
        contourState = None
        self.isContourDisplay = not self.isContourDisplay
        # curCanvas.delete('contourLine')
        if self.isContourDisplay:
            contourState = 'normal'
        else: 
            contourState = 'hidden'
        curCanvas.itemconfig('contourLine', state = contourState)

    
    def wingsButtonOpe(self, curCanvas):
        print('wings button click')
        wingletsState = None
        self.isWingletsDisplay = not self.isWingletsDisplay
        # curCanvas.delete('contourLine')
        if self.isWingletsDisplay:
            wingletsState = 'normal'
        else: 
            wingletsState = 'hidden'
        curCanvas.itemconfig('wingletsLine', state = wingletsState)



import tornado.ioloop
import tornado.options
import tornado.httpserver
import tornado.web

from tornado.options import options,define

import os

setting = dict(
   static_path = os.path.join(os.path.dirname(__file__), './'),
   template_path = os.path.join(os.path.dirname(__file__), './'),
   
)

url = []

application = tornado.web.Application(
    handlers=url,
    debug = True,
    **setting
)

serverPort = 30001
define("port", default=serverPort, help="run on the given port", type=int)

def main():
    tornado.options.parse_command_line()
    http_server = tornado.httpserver.HTTPServer(application)
    print('Development server is running at http://127.0.0.1:%s/' % options.port)
    print('Quit the server with Control-C')
    tornado.ioloop.IOLoop.instance().start()

from handler.handler_all import *

if __name__ == '__main__':
    testName = 'dots'
    outputDots(testName)
    main()
    